/***********************************************************************
 * IBM Confidential 
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2006
 *
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 ************************************************************************/

package com.ibm.cds;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import org.eclipse.osgi.baseadaptor.bundlefile.BundleEntry;
import org.eclipse.osgi.baseadaptor.bundlefile.BundleFile;

/**
 * A bundle entry for a class that is found in the shared classes cache
 */
public class CDSBundleEntry extends BundleEntry {
	String path;
	byte[] classbytes;
	BundleFile wrapped;

	/**
	 * The constructor
	 * @param path the path to the class file
	 * @param classbytes the magic cookie bytes for the class in the shared cache
	 * @param wrapped the actual bundle file where the class comes from
	 */
	public CDSBundleEntry(String path, byte[] classbytes, BundleFile wrapped) {
		super();
		this.path = path;
		this.classbytes = classbytes;
		this.wrapped = wrapped;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.osgi.baseadaptor.bundlefile.BundleEntry#getFileURL()
	 * uses the wrapped bundle file to get the actual file url to the content of
	 * the class on disk.
	 * 
	 * This should is likely never to be called.
	 */
	public URL getFileURL() {
		BundleEntry wrappedEntry = wrapped.getEntry(path);
		if (wrapped == null)
			return null;
		return wrappedEntry.getFileURL();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.osgi.baseadaptor.bundlefile.BundleEntry#getInputStream()
	 * wraps the classbytes into a ByteArrayInputStream.  This should not be used
	 * by classloading.
	 */
	public InputStream getInputStream() throws IOException {
		// someone is trying to get the real bytes of the class file!!
		// just return the entry from the wrapped file instead of the magic cookie
		BundleEntry wrappedEntry = wrapped.getEntry(path);
		if (wrappedEntry != null)
			return wrappedEntry.getInputStream();
		// This should never happen but just in case.
		return new ByteArrayInputStream(classbytes);
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.osgi.baseadaptor.bundlefile.BundleEntry#getBytes()
	 * returns the magic cookie for the shared class.  This is used to define 
	 * the class during class loading.
	 */
	public byte[] getBytes() throws IOException {
		return classbytes;
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.osgi.baseadaptor.bundlefile.BundleEntry#getLocalURL()
	 * uses the wrapped bundle file to get the actual local url to the content of
	 * the class on disk.
	 * 
	 * This should is likely never to be called.
	 */
	public URL getLocalURL() {
		BundleEntry wrappedEntry = wrapped.getEntry(path);
		if (wrapped == null)
			return null;
		return wrappedEntry.getLocalURL();
	}

	public String getName() {
		return path;
	}

	public long getSize() {
		BundleEntry wrappedEntry = wrapped.getEntry(path);
		return wrappedEntry.getSize();
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.osgi.baseadaptor.bundlefile.BundleEntry#getTime()
	 * TODO could use the same approach as getFileURL and getLocalURL here
	 */
	public long getTime() {
		// no time
		return 0;
	}
}
