/***********************************************************************
 * IBM Confidential 
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2006
 *
 * The source code for this program is not published or otherwise divested
 * of its trade secrets, irrespective of what has been deposited with the
 * U.S. Copyright Office.
 ************************************************************************/

package com.ibm.cds;

import java.io.IOException;
import java.net.URL;
import java.security.ProtectionDomain;
import java.util.ArrayList;

import org.eclipse.osgi.baseadaptor.BaseData;
import org.eclipse.osgi.baseadaptor.bundlefile.BundleEntry;
import org.eclipse.osgi.baseadaptor.bundlefile.BundleFile;
import org.eclipse.osgi.baseadaptor.hooks.BundleFileWrapperFactoryHook;
import org.eclipse.osgi.baseadaptor.hooks.ClassLoadingHook;
import org.eclipse.osgi.baseadaptor.hooks.ClassLoadingStatsHook;
import org.eclipse.osgi.baseadaptor.loader.BaseClassLoader;
import org.eclipse.osgi.baseadaptor.loader.ClasspathEntry;
import org.eclipse.osgi.baseadaptor.loader.ClasspathManager;
import org.eclipse.osgi.framework.adaptor.BundleProtectionDomain;
import org.eclipse.osgi.framework.adaptor.ClassLoaderDelegate;

import com.ibm.oti.shared.HelperAlreadyDefinedException;
import com.ibm.oti.shared.Shared;
import com.ibm.oti.shared.SharedClassHelperFactory;
import com.ibm.oti.shared.SharedClassURLHelper;

public class CDSHookImpls implements ClassLoadingStatsHook, ClassLoadingHook, BundleFileWrapperFactoryHook {
	private static SharedClassHelperFactory factory = Shared.getSharedClassHelperFactory();

	////////// ClassLoadingStatsHook /////////
	public void postFindLocalClass(String name, Class clazz, ClasspathManager manager) {
		// do nothing
	}

	public void postFindLocalResource(String name, URL resource, ClasspathManager manager) {
		// do nothing
	}

	public void preFindLocalClass(String name, ClasspathManager manager) throws ClassNotFoundException {
		// do nothing
	}

	public void preFindLocalResource(String name, ClasspathManager manager) {
		// do nothing
	}

	public void recordClassDefine(String name, Class clazz, byte[] classbytes, ClasspathEntry classpathEntry, BundleEntry entry, ClasspathManager manager) {
		// only attempt to record the class define if:
		// 1) the class was found (clazz != null)
		// 2) the class has the magic class number CAFEBABE indicating a real class
		// 3) the bundle file for the classpath entry is of type CDSBundleFile
		if (clazz == null || !hasMagicClassNumber(classbytes) || !(classpathEntry.getBundleFile() instanceof CDSBundleFile))
			return;
		CDSBundleFile cdsFile = (CDSBundleFile) classpathEntry.getBundleFile();
		if (cdsFile.getURL() == null)
			return; // something went wrong trying to determine the url to the real bundle file

		// look for the urlHelper; if it does not exist then we are not sharing for this class loader
		SharedClassURLHelper urlHelper = cdsFile.getURLHelper(); 
		if (urlHelper == null) {
			// this should never happen but just in case get the helper from the base host bundle file.
			BundleFile hostBundleFile = manager.getBaseData().getBundleFile();
			if (hostBundleFile instanceof CDSBundleFile)
				// try getting the helper from the host base cdsFile
				urlHelper = ((CDSBundleFile) hostBundleFile).getURLHelper();
			if (urlHelper != null)
				cdsFile.setURLHelper(urlHelper);
		}
		if (urlHelper != null) {
			// store the class in the cache
			urlHelper.storeSharedClass(cdsFile.getURL(), clazz);
			cdsFile.setPrimed(true);
		}
	}

	private boolean hasMagicClassNumber(byte[] classbytes) {
		if (classbytes == null || classbytes.length < 4)
			return false;
		// TODO maybe there is a better way to do this? I'm not sure why I had to AND each byte with the value I was checking ...
		return (classbytes[0] & 0xCA) == 0xCA && (classbytes[1] & 0xFE) == 0xFE && (classbytes[2] & 0xBA) == 0xBA && (classbytes[3] & 0xBE) == 0xBE;
	}

	////////// ClassLoadingHook /////////
	public BaseClassLoader createClassLoader(ClassLoader parent, ClassLoaderDelegate delegate, BundleProtectionDomain domain, BaseData data, String[] bundleclasspath) {
		// do nothing
		return null;
	}

	public void initializedClassLoader(BaseClassLoader baseClassLoader, BaseData data) {
		// try to get the url helper for this class loader
		if (factory == null)
			return;
		try {
			SharedClassURLHelper urlHelper = factory.getURLHelper((ClassLoader) baseClassLoader);
			// set the url helper for the host base CDSBundleFile
			BundleFile hostFile = data.getBundleFile();
			if (hostFile instanceof CDSBundleFile)
				((CDSBundleFile) hostFile).setURLHelper(urlHelper);

		} catch (HelperAlreadyDefinedException e) {
			// We should never get here. 
			// If we do, we simply won't share for this ClassLoader
		}
	}
	public byte[] processClass(String name, byte[] classbytes, ClasspathEntry classpathEntry, BundleEntry entry, ClasspathManager manager) {
		return null;
	}

	public boolean addClassPathEntry(ArrayList cpEntries, String cp, ClasspathManager hostmanager, BaseData sourcedata, ProtectionDomain sourcedomain) {
		BundleFile hostFile = hostmanager.getBaseData().getBundleFile();
		BundleFile sourceFile = sourcedata.getBundleFile();
		if (hostFile != sourceFile && hostFile instanceof CDSBundleFile && sourceFile instanceof CDSBundleFile)
			// set the helper that got set on the host base bundle file in initializedClassLoader
			((CDSBundleFile) sourceFile).setURLHelper(((CDSBundleFile) hostFile).getURLHelper());
		return false;
	}
	public ClassLoader getBundleClassLoaderParent() {
		// do nothing
		return null;
	}
	public String findLibrary(BaseData data, String libName) {
		// do nothing
		return null;
	}

	//////////////// BundleFileWrapperFactoryHook //////////////
	public BundleFile wrapBundleFile(BundleFile bundleFile, Object content, BaseData data, boolean base) throws IOException {
		// wrap the real bundle file for purposes of loading shared classes.
		if (!base && data.getBundleID() != 0) {
			// initialize the urlHelper from the base one.
			SharedClassURLHelper urlHelper = null;
			BundleFile baseFile = data.getBundleFile();
			if (baseFile instanceof CDSBundleFile)
				urlHelper = ((CDSBundleFile) baseFile).getURLHelper();
			return new CDSBundleFile(bundleFile, urlHelper);
		}
		return new CDSBundleFile(bundleFile);
	}
}
